/**
 * Arduino BayGPRS
 *
 * S.Holzheu (holzheu@bayceer.uni-bayreuth.de)
 *
 * Class for sending HTTP-Post-Request to a BayEOS Gateway
 *
 */

#ifndef BayGPRS_h
#define BayGPRS_h


#ifndef BayGPRS_BUFFER
#define BayGPRS_BUFFER 136
#endif

#define BayGPRS_CONFIG_URL 0
#define BayGPRS_CONFIG_PW 1
#define BayGPRS_CONFIG_SENDER 2
#define BayGPRS_CONFIG_APN 3
#define BayGPRS_CONFIG_PROVPW 4
#define BayGPRS_CONFIG_PROVUSER 5
#define BayGPRS_CONFIG_PIN 6

#define BayGPRS_CONFIG_SIZE 100
#define BayGPRS_EEPROM_OFFSET 400

#define BayGPRS_DEBUG_OUTPUT 0
#define BayGPRS_DEBUG_INPUT 0

#include <inttypes.h>
#include <BayEOS.h>
#include <Base64.h>
#include <HardwareSerial.h>
#include <Arduino.h>
#include "../EEPROM/EEPROM.h"
#include "../BayEOSBuffer/BayEOSBuffer.h"
#include "../SdFat/SdFat.h"

/*
 * RTC-Clock with SIM900 + millis() implementation
 * you have to call adjust() at least every 48 days...
 *
 */

class RTC_SIM900 : public RTC {
public:
    void begin() {}
    void adjust(const DateTime& dt);
    DateTime now();

protected:
    unsigned long offset;
    unsigned long last_millis;
};


class BayGPRSInterface : public Stream, public BayEOS {
protected:
	virtual void i_begin(long) = 0;
	virtual int i_available() = 0;
public:
    using Print::write; // pull in write(str) and write(buf, size) from Print
	/**
	 * Send the BayXBee payload as base64 encoded string
	 * returns 0 for success
	 * 1 == no success
	 * 2 == timeout
	 * 3 == network timeout
	 * 4 == gprs modem timeout
	 */
	uint8_t sendPayload(void);

	/**
	 * Send SMS
	 * Use international phone numbers +49151xxxxx
	 */
	uint8_t sendSMS(const String &phone, const String &sms);

	/**
	 * Connect to the web through apn
	 * 0 == OK
	 * 1 == Network TIMEOUT
	 * 2 == GPRS MODEM TIMEOUT
	 * Omit, if you just want to send SMS
	 */
	uint8_t connect(void);

	/**
	 * Disconnect from the web
	 */
	void disconnect(void);

	/**
	 * Set apn,user,password for internet connection
	 * maxlength for apn is 24, maxlength for user and password is 8
	 */
	void setProvider(const char *apn,const char *user="",const char *pw="");


	/**
	 * Disconnect from the web
	 */
	uint8_t getRSSI(void);

	/**
	 * Set RTC
	 * time has to be in the format "YY/MM/DD,HH:MM:SS-01"
	 */
	uint8_t setClock(const char* time);

	/**
	 * Print line with current time to Serial
	 */
	void serialPrintlnTime(void);

	/**
	 * Set Clock via Serial
	 */
	void serialSetClock(void);

	/**
	 * Get time from RTC
	 */
	DateTime now(void);

	/**
	 * Switch on GPRS-Modem
	 * 0 == GPRS-modem is up and responding OK
	 * 1 == Communication-ERROR
	 * 2 == PIN-Error
	 *
	 */
	virtual uint8_t begin(long baud);

	uint8_t setBaudRate(long baud);


	/**
	 * Send several frames in one post request
	 * returns 0 for success
	 * 1 == no success
	 * 2 == timeout
	 * 3 == network timeout
	 * 4 == gprs modem timeout
	 */
	uint8_t sendMultiFromBuffer(int maxsize=5000);

	/**
	 * set as config value in config buffer
	 */
	void setConfig(const char *str,uint8_t index);
	/**
	 * read config from file (uses SdFat library!)
	 */
	void readConfigFromFile(const char* file);
	/**
	 * read config from EEPROM
	 */
	void readConfigFromEEPROM(int eeoffset=BayGPRS_EEPROM_OFFSET);
	/**
	 * write config to EEPROM
	 */
	void writeConfigToEEPROM(int eeoffset=BayGPRS_EEPROM_OFFSET);
	/**
	 * set as config value via reading form SERIAL
	 */
	void setConfigFromSerial(uint8_t index,uint8_t timeout=100);

	/**
	 * Read date from SERIAL and store it in _SerialInputBuffer
	 */
	void readSerialInput(uint8_t timeout=255);

	/**
	 * get pointer to config buffer
	 * url\0password\0sender\0apn\0prov_pw\0prow_user\0\pin\0
	 */
	const char* getConfig(void){ return _config_buffer; }

	/**
	 * print config to SERIAL
	 */
	void printConfig(void);


	/**
	 * Set buffer for Serial Input
	 * You have to set this, otherwise serialConfig and serialSetClock will work!
	 */
	void setSerialInputBuffer(char* buffer,uint8_t size){
		_SerialInputBuffer=buffer;
		_SerialInputBufferSize=size-1; //one byte for '\0'
	}

	/**
	 * Call serial config menu
	 */
	void serialConfig(void);

	/**
	 * perform soft switch operation
	 * will switch on/off modem
	 */
	void softSwitch(void);

	/**
	 * Send AT command returns
	 * 0 == GPRS-modem is up and responding OK
	 * 1 == GRPS-modem returned something else than OK
	 *
	 */
	uint8_t sendATE0(void);

	void printConfigName(uint8_t index);
protected:
	void setConfigPointers(void);
	uint8_t addToConfigBuffer(uint8_t offset,const char* str);
	char** getConfigPointer(uint8_t index);

	/**
	 * Switch on GPRS-Modem
	 * 0 == GPRS-modem is up and responding OK
	 * 1 == ERROR
	 * 2 == PIN-Error
	 *
	 */
	uint8_t init(void);
	void skipChars(void);
	uint8_t wait_forOK(uint16_t timeout);
	uint8_t wait_forPGM(const char* str, uint16_t timeout,uint8_t bytes=0, char* buffer=NULL);
	uint8_t wait_for_available(uint16_t* timeout,int bytes=1);
	void printPGM(const char *str);
	void printlnPGM(const char *str);
	void serialprintPGM(const char *str);
	void serialprintlnPGM(const char *str);
/*
 * _config_buffer
 * Buffer for config values
 * url\0password\0sender\0apn\0prov_pw\0prow_user\0\pin\0
 */

	uint8_t _config_mode;
	char _config_buffer[BayGPRS_CONFIG_SIZE];
	char _pgm_buffer[20];
	char* _url; //0
	char* _password; //1
	char* _sender; //2
	char* _apn; //3
	char* _prov_pw; //4
	char* _prov_user; //5
	char* _pin; //6
	char _base64buffer[BayGPRS_BUFFER];
	char* _SerialInputBuffer;
	uint8_t _SerialInputBufferSize;
	uint8_t _data_length;
	uint8_t _tx_error_count;
	long _baud;
public:
	uint8_t _powerPin;

};


class BayGPRS : private HardwareSerial, public BayGPRSInterface {
public:
	/**
	 * Constructor
	 */
	BayGPRS(const char *url,const char *pw,const char *sender,const char *apn, HardwareSerial &serial=Serial, uint8_t powerPin=9);
	BayGPRS(HardwareSerial &serial=Serial, uint8_t powerPin=9);
	uint8_t begin(long baud);
private:
	int available(void){return HardwareSerial::available();}
	int read(void){return HardwareSerial::read();}
	void i_begin(long b){ HardwareSerial::begin(b);}
	int i_available(void){return HardwareSerial::available();}
	size_t write(uint8_t b){return HardwareSerial::write(b);}
    int peek(void){return HardwareSerial::peek();};
    void flush(void){HardwareSerial::flush();};
};


#endif
