/*
 * StatusLED Class
 * Class is designed to be used with TimerOne Class
 *
 * use
 * led.init(); //initialize pins
 * Timer1.initialize(80000); //initialize timer to 80ms
 * Timer1.attachInterrupt(led.callback); //attach led callback
 * in setup()
 *
 * to blink use the macros
 * BLINK_RED(times)
 * BLINK_GREEN(times)
 *
 * Alternative there are the methods blink_red, blink_green
 * using delay()
 *
 * STATUS_LED_PIN defaults to 11
 * ERROR_LED_PIN defaults to 12
 *
 * to speed up ISR direct PORT Manipulation is used
 * works only with pin layout of ATMEGA328 and
 * with both LEDs on same port
 * to switch back to digitalWrite define
 * PORT_MANIPULATION 0 before include StatusLED.h
 */
#ifndef StatusLED_h
#define StatusLED_h

/* StatusLED Definitionen */
#define STATUS_LED_XBEE_ERROR 0x1
#define STATUS_LED_XBEE_SUCCESS 0x1
#define STATUS_LED_SENSOR_ERROR 0x2
#define STATUS_LED_SENSOR_SUCCESS 0x2



#include <inttypes.h>
#if ARDUINO >= 100
#include "Arduino.h"       // for delayMicroseconds, digitalPinToBitMask, etc
#else
#include "WProgram.h"      // for delayMicroseconds
#endif


#define DIRECT_MODE_OUTPUT(base, mask)	((*(base+1)) |= (mask))
#define DIRECT_WRITE_LOW(base, mask)	((*(base+2)) &= ~(mask))
#define DIRECT_WRITE_HIGH(base, mask)	((*(base+2)) |= (mask))

#ifndef STATUS_LED_PIN
#define STATUS_LED_PIN 11
#endif

#ifndef ERROR_LED_PIN
#define ERROR_LED_PIN 12
#endif

#define BLINK_RED(times) (led.error_blink=(times) * 4 + 1 )
#define BLINK_GREEN(times) (led.status_blink=(times) * 4 + 1 )

#ifndef PORT_MANIPULATION
#define PORT_MANIPULATION 1
#endif

#if PORT_MANIPULATION==1
#if STATUS_LED_PIN>=8 && ERROR_LED_PIN>=8
#define STATUS_LED_PORT PORTB
#define STATUS_LED_DDR DDRB
#define PIN_OFFSET 8
#endif

#if STATUS_LED_PIN<8 && ERROR_LED_PIN<8
#define STATUS_LED_PORT PORTD
#define STATUS_LED_DDR DDRD
#define PIN_OFFSET 0
#endif

#define ERROR_LED_BIT (ERROR_LED_PIN-PIN_OFFSET)
#define STATUS_LED_BIT (STATUS_LED_PIN-PIN_OFFSET)
#define PIN_MASK ( ~(0x1<<ERROR_LED_BIT | 0x1<<STATUS_LED_BIT) )
#endif

class StatusLED {
public:
	void blink_red(uint8_t times){
		blink(times,ERROR_LED_PIN);
	}
	void blink_green(uint8_t times){
		blink(times,STATUS_LED_PIN);
	};
	void blink_both(uint8_t times){
		blink(times,STATUS_LED_PIN,ERROR_LED_PIN);
	};

volatile static uint8_t status_blink;
volatile static uint8_t error_blink;
static void init(void){
	error_blink=0;
	status_blink=0;
#if PORT_MANIPULATION!=1
		pinMode(STATUS_LED_PIN, OUTPUT);
		pinMode(ERROR_LED_PIN, OUTPUT);
		digitalWrite(STATUS_LED_PIN,0);
		digitalWrite(ERROR_LED_PIN,0);
#endif
#if PORT_MANIPULATION==1
	STATUS_LED_DDR=STATUS_LED_DDR |
			1<<ERROR_LED_BIT |
			1<<STATUS_LED_BIT;
	STATUS_LED_PORT=STATUS_LED_PORT&PIN_MASK;
#endif
	}
static void callback(void){
    if(status_blink){
    	status_blink--;
#if PORT_MANIPULATION!=1
    	if((status_blink & 0x3)>0x1)
    		digitalWrite(STATUS_LED_PIN,(status_blink & 0x3)==0x3);
#endif
    }
    if(error_blink){
    	error_blink--;
#if PORT_MANIPULATION!=1
    	if((error_blink & 0x3)>0x1)
    		digitalWrite(ERROR_LED_PIN, (error_blink & 0x3)==0x3);
#endif
    }
#if PORT_MANIPULATION==1
 	STATUS_LED_PORT=STATUS_LED_PORT&PIN_MASK |
 			((error_blink & 0x3)==0x3)<<ERROR_LED_BIT |
			((status_blink & 0x3)==0x3)<<STATUS_LED_BIT;
#endif
}

private:
	void blink(uint8_t times,uint8_t pin1, uint8_t pin2=0);

};

extern StatusLED led;




#endif
